/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * GUI functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>

#include <parted/parted.h>
#include <gtk/gtk.h>
//#include <gdk/gdkkeysyms.h>

#include "../misc/misc.h"

#include "gped.h"
#include "fstab.h"
#include "ctreeParted.h"
#include "ctreeParted_callbacks.h"


/* pixmaps */
#include "../pixmaps/i-blockdev-disk.xpm"
#include "../pixmaps/i-blockdev-primary.xpm"
#include "../pixmaps/i-blockdev-extended.xpm"
#include "../pixmaps/i-blockdev-logical.xpm"

#define PERCENTAGE_PIXMAP_WIDTH 70 
#define PERCENTAGE_PIXMAP_HEIGHT 15

/*
 * create percentage pixmap
 */
static GdkPixmap*
percentage_pixmap_create              (GtkCTree        *ctreeParted,
                                       PedPartition    *part)
{
	GdkPixmap       *pixmap;
	GdkGC		*gc; 
	GdkFont		*font;
	GdkColor	 colors[3];
	gboolean	 succes[3];
	GdkColormap	*colormap;
	gushort		 bar_start;  
	gushort		 bar_length;  
	gchar		*text;
	
	colormap = gdk_window_get_colormap (GTK_WIDGET (ctreeParted)->window);
	pixmap = gdk_pixmap_new (GTK_WIDGET (ctreeParted)->window,
	                         PERCENTAGE_PIXMAP_WIDTH,
	                         PERCENTAGE_PIXMAP_HEIGHT,
	                         -1);
	gc = gdk_gc_new (pixmap);
	
	/* foreground */
	colors[0].red = 0;
	colors[0].green = 0;
	colors[0].blue = 40191;
	/* background */
	colors[1].red = 26367;
	colors[1].green = 39424;
	colors[1].blue = 65535;
	/* text */
	colors[2].red = 65535;
	colors[2].green = 65535;
	colors[2].blue = 65535;
	gdk_colormap_alloc_colors (colormap,
	                           colors, 3,
	                           FALSE, TRUE,
	                           succes);
	
	gdk_gc_set_foreground (gc, &colors[0]);
	gdk_draw_rectangle (pixmap, gc, 1,
	                    0, 0,
	                    PERCENTAGE_PIXMAP_WIDTH,
	                    PERCENTAGE_PIXMAP_HEIGHT);
	
	bar_start = PERCENTAGE_PIXMAP_WIDTH
	              * part->geom.start / part->geom.disk->dev->length;
	bar_length = PERCENTAGE_PIXMAP_WIDTH
	               * part->geom.length / part->geom.disk->dev->length;

	gdk_gc_set_foreground (gc, &colors[1]);
	gdk_draw_rectangle (pixmap, gc, 1,
	                    bar_start, 0,
	                    bar_length,
	                    PERCENTAGE_PIXMAP_HEIGHT);
	
	font = GTK_WIDGET (ctreeParted)->style->font;
	gdk_gc_set_font (gc, font);
	gdk_gc_set_foreground (gc, &colors[2]);
	text = g_strdup_printf (_("%i%%"), (int) (100 * part->geom.length
	                                           / part->geom.disk->dev->length));
	gdk_draw_string (pixmap, font, gc,
	                 (PERCENTAGE_PIXMAP_WIDTH-gdk_string_width (font, text))/2,
	                 (PERCENTAGE_PIXMAP_HEIGHT+font->ascent-font->descent)/2,
	                 text);
	g_free (text);
	
	gdk_gc_destroy (gc);
	
	return pixmap;
}


static gchar*
strdup_partition_flags_list           (PedPartition    *part)
{
	PedPartitionFlag	 flag = 0;
	gchar			*buffer = NULL;

	buffer = g_strdup ("");	
	while ( (flag = ped_partition_flag_next (flag)) ) {
		if ( ped_partition_get_flag (part, flag) )
			buffer = g_strconcat (buffer,
			                      " ",
			                      ped_partition_flag_get_name (flag),
			                      NULL);
	}
	
	return buffer;
}

/*
 * insert partition nodes in ctreeParted
 */
static gboolean
_insert_partitions_nodes               (GtkCTree        *ctreeParted,
                                        GtkCTreeNode    *father,
                                        PedDisk         *disk,
                                        gpedFSTabInfo	*fstab_info)
{
	PedPartition	*part = NULL;
	GtkCTreeNode	*father_extended = NULL;

	g_return_val_if_fail (ctreeParted, FALSE);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), FALSE);
	g_return_val_if_fail (father, FALSE);
	g_return_val_if_fail (disk, FALSE);
	g_return_val_if_fail (fstab_info, FALSE);

	while ( (part = ped_disk_next_partition (disk, part)) ) {  /* part in disk*/
		GtkCTreeNode	*node = NULL;
 		gchar		*node_texts[6];
		unsigned int	 i;
		static GdkPixmap	*primary = NULL;
		static GdkBitmap	*primary_mask = NULL;
		static GdkPixmap	*extended = NULL;
		static GdkBitmap	*extended_mask = NULL;
		static GdkPixmap	*logical = NULL;
		static GdkBitmap	*logical_mask = NULL;
		GdkPixmap	*percentage;
		
		if (part->type != PED_PARTITION_PRIMARY
		    && part->type != PED_PARTITION_LOGICAL
		    && part->type != PED_PARTITION_EXTENDED
		    && !(part->type & PED_PARTITION_FREESPACE))
			continue;
		
		/*
		 * initialize ctree line texts
		 */
		if (part->type & PED_PARTITION_FREESPACE) {
			node_texts[0] = g_strdup (_("Free Space"));
			node_texts[1] = g_strdup ("");
			node_texts[2] = g_strdup ("");	/* will be replaced by a pixmap */
			node_texts[3] = g_strdup ("");
			node_texts[4] = g_strdup ("");
			node_texts[5] = g_strdup ("");
		} else {	
			node_texts[0] = g_strdup_printf ("%i", part->num);
			node_texts[1] = g_strdup (part->fs_type ? part->fs_type->name : _("unknown"));
			node_texts[2] = g_strdup_printf (_("%i MB"), (int) (part->geom.length * disk->dev->sector_size / one_MB));
			node_texts[3] = g_strdup ("");	/* will be replaced by a pixmap */
			node_texts[4] = strdup_partition_flags_list (part);
			node_texts[5] = g_strdup (fstab_get_part_mount_point (fstab_info, part));
		}
			
		/*
		 * insert node
		 */ 
		if (part->type == PED_PARTITION_PRIMARY) {
			ui_pixmap_init (GTK_WIDGET (ctreeParted), &primary, &primary_mask, i_blockdev_primary_xpm);
			
			node = gtk_ctree_insert_node (GTK_CTREE (ctreeParted),
			                              father, NULL,
			                              node_texts, 5,
			                              primary, primary_mask,
			                              primary, primary_mask,
			                              TRUE, FALSE);
		} else if (part->type == PED_PARTITION_EXTENDED && !father_extended) {
			ui_pixmap_init (GTK_WIDGET (ctreeParted), &extended, &extended_mask, i_blockdev_extended_xpm);
			
			father_extended = node = gtk_ctree_insert_node (GTK_CTREE (ctreeParted),
			                              father, NULL,
			                              node_texts, 5,
			                              extended, extended_mask,
			                              extended, extended_mask,
			                              FALSE, TRUE);
		} else if (part->type == PED_PARTITION_LOGICAL
		             && father_extended) {
			ui_pixmap_init (GTK_WIDGET (ctreeParted), &logical, &logical_mask, i_blockdev_logical_xpm);
		
			node = gtk_ctree_insert_node (GTK_CTREE (ctreeParted),
			                              father_extended, NULL,
			                              node_texts, 5,
			                              logical, logical_mask,
			                              logical, logical_mask,
			                              TRUE, FALSE);
		} else if (part->type & PED_PARTITION_LOGICAL
		             && part->type & PED_PARTITION_FREESPACE
		             && father_extended) {
			node = gtk_ctree_insert_node (GTK_CTREE (ctreeParted),
			                              father_extended, NULL,
			                              node_texts, 5,
			                              NULL, NULL,
			                              NULL, NULL,
			                              TRUE, FALSE);
		} else if (part->type == PED_PARTITION_FREESPACE) {
			node = gtk_ctree_insert_node (GTK_CTREE (ctreeParted),
			                              father, NULL,
			                              node_texts, 5,
			                              NULL, NULL,
			                              NULL, NULL,
			                              TRUE, FALSE);
		}else { /* should never go there */
			g_assert_not_reached ();
		}
		
		/* free text memory usage */
		for (i=0; i<6; i++)
			g_free(node_texts[i]);
		
		percentage = percentage_pixmap_create (GTK_CTREE (ctreeParted),
		                                       part);
		gtk_ctree_node_set_pixmap (GTK_CTREE (ctreeParted),
		                           GTK_CTREE_NODE (node),
		                           3, percentage, NULL);
		gdk_pixmap_unref (percentage);
		
		/* attach PedPartition to node */
		gtk_ctree_node_set_row_data(GTK_CTREE (ctreeParted), GTK_CTREE_NODE (node), part);
	}

	return TRUE;
}

/*
 * insert disk node in ctreeParted
 */
static gboolean
_insert_device_node                   (GtkCTree         *ctreeParted,
                                       gpedDeviceInfo   *devInfo,
                                       gpedFSTabInfo	*fstab_info)
{
	GtkCTreeNode	*father;
	gchar		*node_texts[6];
	unsigned int	 i;
	static GdkPixmap	*hd = NULL;
	static GdkBitmap	*hd_mask = NULL;

	g_return_val_if_fail (ctreeParted, FALSE);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), FALSE);
	g_return_val_if_fail (devInfo, FALSE);
	g_return_val_if_fail (devInfo->dev, FALSE);
	g_return_val_if_fail (fstab_info, FALSE);
	
	
	/*
	 * initialize ctree line texts
	 */
	node_texts[0] = g_strdup (devInfo->dev->path);
	node_texts[1] = g_strdup ("");
	node_texts[2] = g_strdup_printf (_("%i MB"), (int) (devInfo->dev->length * devInfo->dev->sector_size / one_MB));
	node_texts[3] = g_strdup ("");
	node_texts[4] = g_strdup ("");
	node_texts[5] = g_strdup ("");
	
	/* hd pixmap initialization */
	ui_pixmap_init (GTK_WIDGET (ctreeParted), &hd, &hd_mask, i_blockdev_disk_xpm);

	/*
	 * insert root node for disk
	 */ 
	father = gtk_ctree_insert_node (GTK_CTREE (ctreeParted),
	                       NULL, NULL,
	                       node_texts, 5,
	                       hd,
	                       hd_mask,
	                       hd,
	                       hd_mask,
	                       FALSE, TRUE);
	/* free text memory usage */
	for (i=0; i<6 ; i++)
		g_free(node_texts[i]);
			
	/* attach gpedDeviceInfo devInfo to node */
	gtk_ctree_node_set_row_data(GTK_CTREE (ctreeParted), GTK_CTREE_NODE (father), devInfo);
	
	/*
	 * insert child nodes (ie: partition) for disk
	 */	
	if (devInfo->disk) {
		if ( !_insert_partitions_nodes (GTK_CTREE (ctreeParted),GTK_CTREE_NODE (father), devInfo->disk, fstab_info) )
		 return FALSE;
	}
	return TRUE;
}

/*
 * insert root (disk) nodes in ctreeParted
 */
gboolean
_insert_devices_nodes                 (GtkCTree         *ctreeParted,
                                       gpedDeviceInfo   *dev_list,
                                       gpedFSTabInfo	*fstab_info)
{
	gpedDeviceInfo	*walk;

	g_return_val_if_fail (ctreeParted, FALSE);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), FALSE);
	g_return_val_if_fail (dev_list, FALSE); 
	g_return_val_if_fail (fstab_info, FALSE);
	
	for (walk = dev_list; walk; walk = walk->next) {
		if ( !_insert_device_node (GTK_CTREE (ctreeParted), walk, fstab_info) )
			return FALSE;
	}

	return TRUE;
}

/*
 * delete all nodes and recreate them
 */
void
ctreeParted_refresh                    (GtkCTree        *ctreeParted,
                                        gpedInfo        *info)
{
	g_return_if_fail (ctreeParted);
	g_return_if_fail (GTK_IS_CTREE (ctreeParted));
	g_return_if_fail (info);
	g_return_if_fail (info->dev_list);
	g_return_if_fail (info->fstab_info);

	gtk_clist_freeze (GTK_CLIST (ctreeParted));
	gtk_ctree_post_recursive (GTK_CTREE (ctreeParted),
	                    NULL,
	                    GTK_CTREE_FUNC (gtk_ctree_remove_node),
	                    NULL);
	_insert_devices_nodes (GTK_CTREE (ctreeParted),
	                       info->dev_list,
	                       info->fstab_info);
	gtk_clist_thaw (GTK_CLIST (ctreeParted));
}

GtkWidget*
ctreeParted_insert                     (GtkContainer    *container,
                                        gpedInfo        *info)
{
	GtkWidget	*ctreeParted;
	gchar		*ctree_titles[] = {_("Device"),_("FileSystem"),_("Size"),_("Use"),_("Flags"),_("Mount Point")};

	g_return_val_if_fail (container, NULL);
	g_return_val_if_fail (GTK_IS_CONTAINER (container), NULL);
	g_return_val_if_fail (info, NULL);
	g_return_val_if_fail (info->dev_list, NULL);
	
	ctreeParted = gtk_ctree_new_with_titles (6, 0, ctree_titles);
	gtk_container_add (GTK_CONTAINER (container), ctreeParted);
	gtk_clist_column_titles_passive(GTK_CLIST (ctreeParted));
	_insert_devices_nodes (GTK_CTREE (ctreeParted),
	                       info->dev_list,
	                       info->fstab_info);
	gtk_clist_columns_autosize (GTK_CLIST (ctreeParted));
//	gtk_clist_optimal_column_width (GTK_CLIST (ctreeParted), 1);
	
	gtk_clist_set_reorderable (GTK_CLIST (ctreeParted), TRUE);
	
	gtk_signal_connect (GTK_OBJECT (ctreeParted),
	                    "tree-move",
	                    GTK_SIGNAL_FUNC (on_ctreeParted_tree_move),
	                    info);

	return ctreeParted;
}
