/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <parted/parted.h>
#include <gtk/gtk.h>

#include "../misc/misc.h"

#include "gped.h"
#include "fstab.h"
#include "dialogOp.h"
#include "dialogOp_private.h"
#include "dialogOp_callbacks.h"

/*
 * dialogOp save functions
 */

/*
 * partition must be already in disk
 * ped_disk_set_partition_geom needs it
 */ 
static gboolean
dialogOp_save_size                     (gpedOpInfo       *op)
{
	PedConstraint	*constraint;
	PedSector	 start;
	PedSector	 end;
	
	g_return_val_if_fail (op, FALSE);
	
	start = (PedSector) gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (op->spinbuttonStart));
	end = (PedSector) gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (op->spinbuttonEnd));

	constraint = ped_constraint_any (op->disk);
	if (!ped_disk_set_partition_geom (op->disk, op->part,
	                                  constraint, start, end))
		goto error_free_constraint;
	ped_constraint_destroy (constraint);
	
	gped_change_done (op->info, op->disk);
	return TRUE;

error_free_constraint:
	ped_constraint_destroy (constraint);
	return FALSE;
}



static gboolean
_save_flag                             (PedPartition     *part,
                                        PedPartitionFlag  flag,
                                        gboolean          new_state)
{
	g_return_val_if_fail (part, FALSE);	
	g_return_val_if_fail (flag, FALSE);

	if ( !ped_partition_is_flag_available (part, flag) )
		return TRUE;
	
	if ( ped_partition_get_flag (part, flag) !=  new_state )
		if ( !ped_partition_set_flag (part, flag, new_state))
			goto error;

	return TRUE;

error:
	return FALSE;
}

static void
dialogOp_save_flags                    (gpedOpInfo       *op)
{
	PedPartitionFlag	flag = 0;

	g_return_if_fail (op);
	
	/*
	 * set partition flags
	 */
	while ( (flag = ped_partition_flag_next (flag)) ) {
		g_return_if_fail (flag <= PED_PARTITION_LAST_FLAG);
		g_return_if_fail (op->checkbuttonFlags[flag-1]);
		g_return_if_fail (GTK_IS_CHECK_BUTTON (op->checkbuttonFlags[flag-1]));		
		
		_save_flag (op->part, flag,
		            gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (op->checkbuttonFlags[flag-1])));
	}
}

static void
_add_new_fstab_list_entry              (gpedOpInfo       *op,
                                        PedPartition     *part)
{
	g_return_if_fail (op);
	g_return_if_fail (op->entryMount);
	g_return_if_fail (op->info);
	g_return_if_fail (op->info->fstab_info);
	g_return_if_fail (part);

	fstab_append_new_entry (op->info->fstab_info,
	                        part,
	                        g_strdup (gtk_entry_get_text (GTK_ENTRY (op->entryMount))),
	                        NULL);
	fstab_change_done (op->info);
}

static void
_update_fstab_list_entry               (gpedOpInfo       *op,
                                        PedPartition     *part) 
{
	gpedFSTabInfo	*fstab_info;
	gchar		*current_mount_point;
	gchar		*new_mount_point;	
	
	g_return_if_fail (op);
	g_return_if_fail (op->entryMount);
	g_return_if_fail (op->info);
	g_return_if_fail (op->info->fstab_info);
	g_return_if_fail (part);
	
	fstab_info = op->info->fstab_info;
	current_mount_point = fstab_get_part_mount_point (fstab_info, part);
	new_mount_point =  gtk_entry_get_text (GTK_ENTRY (op->entryMount));

	if ( !str_are_equal (current_mount_point, new_mount_point) ) {
		if ( !fstab_set_part_mount_point (fstab_info,
		                                  part,
		                                  g_strdup (new_mount_point)) )
			_add_new_fstab_list_entry (op, part);
		fstab_change_done (op->info);
	}
}

/*
 * dialogProperties apply partitions changes
 */
void
on_dialogProperties_buttonOk_clicked   (GtkButton        *button,
                                        gpedPropInfo     *prop)
{
	gboolean	 resize = FALSE;
	gboolean	 resizeFS = FALSE;
	PedFileSystem	*fs = NULL;

	g_return_if_fail (button);
	g_return_if_fail (GTK_IS_BUTTON (button));
	g_return_if_fail (prop);
	g_return_if_fail (GPED_OP_INFO (prop)->part);

	dialogOp_save_flags (GPED_OP_INFO (prop));

	if (TRUE) {
		resize = TRUE;
		resizeFS = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (prop->checkbuttonResizeCreateFS)); 
	}
	
	if (resize && resizeFS) {
		if ( gped_get_disk_changes (GPED_OP_INFO (prop)->info, GPED_OP_INFO (prop)->disk)  ) {	/* ask to write changes */
			dialog_option	 ret;
			
			ret = ui_dialog_opt_create_and_run (_("Disk changes must be written to resize FileSystem !"),
			                                    DIALOG_OK_CANCEL);
			if (ret == DIALOG_CANCEL)
				return;
		}
		
		if (!GPED_OP_INFO (prop)->part->fs_type)
			g_assert_not_reached ();
			
		/*
		 * open fs before resizing part
		 */
		if (!(fs = ped_file_system_open (&GPED_OP_INFO (prop)->part->geom)) )
			return;
	}
	
	/*
	 * resize part
	 */
	if (resize) {
		if ( !dialogOp_save_size (GPED_OP_INFO (prop)) ) {
			ped_file_system_close (fs);
			return;
		}
	}

	/*
	 * resize FS
	 */
	if (resize && resizeFS) {
		GtkWidget *popup;
		
		gped_write_disk_changes (GPED_OP_INFO (prop)->info, GPED_OP_INFO (prop)->disk);
		
		popup = ui_dialogPopup_create (_("Resizing FileSystem,\n please wait..."));
		gtk_thread_inc_ref_count ();
		
		if (!ped_file_system_resize (fs, &GPED_OP_INFO (prop)->part->geom)) {
			gtk_thread_dec_ref_count();
			gtk_widget_destroy (GTK_WIDGET (popup));
			ped_file_system_close (fs);
			return;
		}
		
		gtk_thread_dec_ref_count ();
		gtk_widget_destroy (GTK_WIDGET (popup));
		
		if (!ped_file_system_close (fs))
			return;
	}

	/*
	 * save mount point
	 */
	_update_fstab_list_entry (GPED_OP_INFO (prop),
	                          GPED_OP_INFO (prop)->part);
}


/*
 * save new partition
 */
void
on_dialogAdd_buttonOk_clicked          (GtkButton        *button,
                                        gpedAddInfo      *add)
{
	PedPartition	*part;
	PedConstraint	*constraint;
	gboolean	 createFS;
	
	g_return_if_fail (button);
	g_return_if_fail (GTK_IS_BUTTON (button));
	g_return_if_fail (add);
	g_return_if_fail (GPED_OP_INFO (add));
	g_return_if_fail (GPED_OP_INFO (add)->disk);
	g_return_if_fail ( (part = GPED_OP_INFO (add)->part) );

	/*
	 * first try to add partition
	 */
	constraint = ped_constraint_any (part->geom.disk);
	if ( !ped_disk_add_partition (part->geom.disk, part, constraint) )
		goto error_free_constraint;
	ped_constraint_destroy (constraint);
	gped_change_done (GPED_OP_INFO (add)->info, GPED_OP_INFO (add)->disk);
	
	if ( !dialogOp_save_size (GPED_OP_INFO (add)) )
		goto error_free_part;
	
	dialogOp_save_flags (GPED_OP_INFO (add));
	
	createFS = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (add->checkbuttonCreateFS));
	if ( createFS ) {
		GtkWidget		*popup;
		PedFileSystem		*fs;
		PedFileSystemType	*fs_type;
		gchar			*string_type;
		
		if ( ui_dialog_opt_create_and_run (_("Disk changes must be written before creating FileSystem !"),
				                   DIALOG_OK_CANCEL) == DIALOG_CANCEL )
			return;
		gped_write_disk_changes (GPED_OP_INFO (add)->info,
		                         part->geom.disk);
		
		string_type = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (GPED_OP_INFO (add)->comboSystemType)->entry));
		if ( !(fs_type = ped_file_system_type_get (string_type)) )
			g_assert_not_reached ();
		
		popup = ui_dialogPopup_create (_("Creating FileSystem,\n please wait..."));
		gtk_thread_inc_ref_count ();
		
		if ( !(fs = ped_file_system_create (&part->geom, fs_type)) ) {
			gtk_thread_dec_ref_count ();
			gtk_widget_destroy (GTK_WIDGET (popup));
			goto error;
		}
		
		gtk_thread_dec_ref_count ();
		gtk_widget_destroy (GTK_WIDGET (popup));
	
		ped_file_system_close (fs);
	}

	/*
	 * save mount point
	 */
	_add_new_fstab_list_entry (GPED_OP_INFO (add),
	                           part);
	
	
	return;

error_free_constraint:
	ped_constraint_destroy (constraint);
error_free_part:
	ped_partition_destroy (GPED_OP_INFO (add)->part);
error:
	return;
}


/*
 * free memory used by dialogProperties
 */
void
on_dialogProperties_destroyed          (GtkWindow        *dialogProperties,
                                        gpedPropInfo     *prop)
{
	g_return_if_fail (dialogProperties);
	g_return_if_fail (GTK_IS_WINDOW (dialogProperties));
	g_return_if_fail (prop);
	
	g_free (prop);
}

/*
 * free memory used by dialogAdd
 */
void
on_dialogAdd_destroyed                 (GtkWindow        *dialogAdd,
                                        gpedAddInfo      *add)
{
	g_return_if_fail (dialogAdd);
	g_return_if_fail (GTK_IS_WINDOW (dialogAdd));
	g_return_if_fail (add);
	
	g_free (add);
}


/*
 * dialog Properties & Add callback functions
 */

/*
 * constrain resize Adjustment to possible values for FS
 * in dialogProperties
 */
void
on_checkbuttonResizeFS_toggled         (GtkToggleButton  *checkbuttonResizeFS,
                                        gpedPropInfo     *prop)
{
	GtkAdjustment	*spinbuttonSize_adj;
	gpedOpInfo	*op;
	
	g_return_if_fail (checkbuttonResizeFS);
	g_return_if_fail (GTK_IS_TOGGLE_BUTTON (checkbuttonResizeFS));
	g_return_if_fail (prop);
	
	op = GPED_OP_INFO (prop);
	
	if ( !(spinbuttonSize_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonSize))) )
		return;

	
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbuttonResizeFS)) ) {
		PedFileSystem	*fs;
		PedConstraint	*constraint;
		
		if (!op->part->fs_type)
			g_assert_not_reached ();	/* should never arrive */
		
		if ( !(fs = ped_file_system_open (&op->part->geom)) )
			return;
		
		constraint = ped_file_system_get_resize_constraint (fs);
		spinbuttonSize_adj->lower = constraint->min_size * op->disk->dev->sector_size / one_MB;
		ped_constraint_destroy (constraint);
		
		if ( !ped_file_system_close (fs) )
			return;
	
		if ( !strcmp (op->part->fs_type->name, "ext2") ) {
			GtkAdjustment	*spinbuttonStart_adj;
			
			if ( !(spinbuttonStart_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonStart))) )
				return;
			spinbuttonStart_adj->value =  op->part->geom.start;
			
			gtk_widget_set_sensitive (GTK_WIDGET (op->labelStart), FALSE);
			gtk_widget_set_sensitive (GTK_WIDGET (op->spinbuttonStart), FALSE);
		}
	} else {
			spinbuttonSize_adj->lower = 1;
			gtk_widget_set_sensitive (GTK_WIDGET (op->labelStart), TRUE);
			gtk_widget_set_sensitive (GTK_WIDGET (op->spinbuttonStart), TRUE);
	}
	
	gtk_spin_button_update (GTK_SPIN_BUTTON (op->spinbuttonSize));
}

/*
 * update checkbutton flags availibility on fs_type change
 */
void
on_comboSystemType_changed           (GtkEntry         *comboSystemType_entry,
                                      gpedOpInfo       *op)
{
	PedPartitionFlag	 flag = 0;
	gchar			*fs_name;
	
	g_return_if_fail (comboSystemType_entry);
	g_return_if_fail (GTK_IS_ENTRY (comboSystemType_entry));
	g_return_if_fail ( (fs_name = gtk_entry_get_text (GTK_ENTRY (comboSystemType_entry))) );
	g_return_if_fail (op);
	g_return_if_fail (op->checkbuttonFlags);
	g_return_if_fail (op->part);
	
	/*
	 * change fs type
	 */
	if ( !ped_partition_set_system (op->part,
	                                ped_file_system_type_get (fs_name)) )
		return;
	
	/*
	 * change checkbutton flags availibility
	 */
	while ( (flag = ped_partition_flag_next (flag)) ) {
		g_return_if_fail (flag <= PED_PARTITION_LAST_FLAG);
		g_return_if_fail (op->checkbuttonFlags[flag-1]);
		g_return_if_fail (GTK_IS_CHECK_BUTTON (op->checkbuttonFlags[flag-1]));

		gtk_widget_set_sensitive (GTK_WIDGET (op->checkbuttonFlags[flag-1]),
		                          ped_partition_is_flag_available (op->part, flag));
	}
}


/*
 * GtkSpinButton callbacks
 */


/*
 * block recursive calls to on_spinbutton*_value_changed
 */
static gboolean	block_signal;

/*
 * updates other GtkSpinButton when a value is changed
 */
void
on_spinbuttonSize_adj_value_changed    (GtkAdjustment    *spinbuttonSize_adj,
                                        gpedOpInfo       *op)
{	
	GtkAdjustment	*spinbuttonStart_adj;
	GtkAdjustment	*spinbuttonEnd_adj;
	PedSector	 length;
	PedSector	 end;
	
	g_return_if_fail (spinbuttonSize_adj);
	g_return_if_fail (GTK_IS_ADJUSTMENT (spinbuttonSize_adj));
	g_return_if_fail (op);
	if (block_signal)
		return;
	
	spinbuttonStart_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonStart));
	spinbuttonEnd_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonEnd));
	
	length = spinbuttonSize_adj->value * one_MB / op->disk->dev->sector_size;
	
	end = spinbuttonStart_adj->value + length;
	block_signal = TRUE;
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (op->spinbuttonEnd),
	                           (gfloat) end);
	block_signal = FALSE;

	if (end > (gint) spinbuttonEnd_adj->value) { /* end does not clamp to possible values */
		gint start;
		
		start = spinbuttonStart_adj->value - end + spinbuttonEnd_adj->value;
		block_signal = TRUE;
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (op->spinbuttonStart),
		                          (gfloat) start);
		block_signal = FALSE;
	}
}

static void
adjust_spinbuttonSize                  (gpedOpInfo       *op)
{
	GtkAdjustment	*spinbuttonStart_adj;
	GtkAdjustment	*spinbuttonEnd_adj;
	guint		 size;
	PedSector	 length;
	
	g_return_if_fail (op);
	if (block_signal)	/* signal should be blocked before */
		g_assert_not_reached ();
	
	spinbuttonStart_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonStart));
	spinbuttonEnd_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonEnd));

	length = spinbuttonEnd_adj->value - spinbuttonStart_adj->value;
	size = length * op->disk->dev->sector_size / one_MB;

	block_signal = TRUE;
	gtk_spin_button_set_value (GTK_SPIN_BUTTON (op->spinbuttonSize),
	                           (gfloat) size);
	block_signal = FALSE;
}

void
on_spinbuttonStart_adj_value_changed   (GtkAdjustment    *spinbuttonStart_adj,
                                        gpedOpInfo       *op)
{	
	GtkAdjustment	*spinbuttonEnd_adj;

	g_return_if_fail (spinbuttonStart_adj);
	g_return_if_fail (GTK_IS_ADJUSTMENT (spinbuttonStart_adj));
	g_return_if_fail (op);
	if (block_signal)
		return;

	spinbuttonEnd_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonEnd));
	
	if (spinbuttonStart_adj->value > spinbuttonEnd_adj->value) {
		block_signal = TRUE;
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (op->spinbuttonStart),
		                           spinbuttonEnd_adj->value);
		block_signal = FALSE;
	}

	adjust_spinbuttonSize (op);
}

void
on_spinbuttonEnd_adj_value_changed     (GtkAdjustment    *spinbuttonEnd_adj,
                                        gpedOpInfo       *op)
{	
	GtkAdjustment	*spinbuttonStart_adj;

	g_return_if_fail (spinbuttonEnd_adj);
	g_return_if_fail (GTK_IS_ADJUSTMENT (spinbuttonEnd_adj));
	g_return_if_fail (op);
	if (block_signal)
		return;

	spinbuttonStart_adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON (op->spinbuttonStart));
	
	if (spinbuttonEnd_adj->value < spinbuttonStart_adj->value) {
		block_signal = TRUE;
		gtk_spin_button_set_value (GTK_SPIN_BUTTON (op->spinbuttonEnd),
		                           spinbuttonStart_adj->value);
		block_signal = FALSE;
	}

	adjust_spinbuttonSize (op);
}
